Index: apps/app_talkdetect.c
===================================================================
--- apps/app_talkdetect.c	(revision 106141)
+++ apps/app_talkdetect.c	(working copy)
@@ -44,15 +44,16 @@
 static char *synopsis = "Background a file with talk detect";
 
 static char *descrip = 
-"  BackgroundDetect(filename[,sil[,min,[max]]]):  Plays  back  a  given\n"
+"  BackgroundDetect(filename[,sil[,min,[max,[analysistime]]]]):  Plays  back  a  given\n"
 "filename, waiting for interruption from a given digit (the digit must\n"
 "start the beginning of a valid extension, or it will be ignored).\n"
 "During the playback of the file, audio is monitored in the receive\n"
 "direction, and if a period of non-silence which is greater than 'min' ms\n"
-"yet less than 'max' ms is followed by silence for at least 'sil' ms then\n"
+"yet less than 'max' ms is followed by silence for at least 'sil' ms\n"
+"which occurs before the 'analysistime' ms then\n"
 "the audio playback is aborted and processing jumps to the 'talk' extension\n"
-"if available.  If unspecified, sil, min, and max default to 1000, 100, and\n"
-"infinity respectively.\n";
+"if available.  If unspecified, sil, min, max, analysistime default to 1000, 100,\n"
+"infinity, and infinity respectively.\n";
 
 
 static int background_detect_exec(struct ast_channel *chan, void *data)
@@ -62,9 +63,12 @@
 	struct ast_frame *fr;
 	int notsilent = 0;
 	struct timeval start = { 0, 0};
+	struct timeval chanstart = { 0, 0};
 	int sil = 1000;
 	int min = 100;
 	int max = -1;
+	int analysistime = -1;
+	int analysis = 1;
 	int x;
 	int origrformat=0;
 	struct ast_dsp *dsp = NULL;
@@ -73,6 +77,7 @@
 		AST_APP_ARG(silence);
 		AST_APP_ARG(min);
 		AST_APP_ARG(max);
+		AST_APP_ARG(analysistime);
 	);
 	
 	if (ast_strlen_zero(data)) {
@@ -89,8 +94,10 @@
 		min = x;
 	if (!ast_strlen_zero(args.max) && (sscanf(args.max, "%d", &x) == 1) && (x > 0))
 		max = x;
+	if (!ast_strlen_zero(args.analysistime) && (sscanf(args.analysistime, "%d", &x) == 1) && (x > 0))
+		analysistime = x;
 
-	ast_debug(1, "Preparing detect of '%s', sil=%d, min=%d, max=%d\n", args.filename, sil, min, max);
+	ast_debug(1, "Preparing detect of '%s', sil=%d, min=%d, max=%d, analysistime=%d\n", args.filename, sil, min, max, analysistime);
 	do {
 		if (chan->_state != AST_STATE_UP) {
 			if ((res = ast_answer(chan)))
@@ -114,7 +121,7 @@
 			ast_log(LOG_WARNING, "ast_streamfile failed on %s for %s\n", chan->name, (char *)data);
 			break;
 		}
-
+		chanstart = ast_tvnow();
 		while (chan->stream) {
 			res = ast_sched_wait(chan->sched);
 			if ((res < 0) && !chan->timingfunc) {
@@ -129,6 +136,15 @@
 				break;
 			} else if (res > 0) {
 				fr = ast_read(chan);
+				if (analysis && analysistime >= 0){
+				    /* If we have a limit for the time to analyze voice
+				    frames and the time has not expired */
+				    if (ast_tvdiff_ms(ast_tvnow(), chanstart) >= analysistime){
+					analysis = 0;
+					ast_verb(3, "BackgroundDetect: Channel [%s]. Talk analysis time complete\n", chan->name);
+				    }
+				}
+				
 				if (!fr) {
 					res = -1;
 					break;
@@ -142,7 +158,7 @@
 						ast_frfree(fr);
 						break;
 					}
-				} else if ((fr->frametype == AST_FRAME_VOICE) && (fr->subclass == AST_FORMAT_SLINEAR)) {
+				} else if ((fr->frametype == AST_FRAME_VOICE) && (fr->subclass == AST_FORMAT_SLINEAR) && analysis) {
 					int totalsilence;
 					int ms;
 					res = ast_dsp_silence(dsp, fr, &totalsilence);
